/*
 * A memory card region accessor.
 *
 * Copyright (C) 2021, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <string.h>

#include <algorithm>

#include "msc_region_accessor.h"



MscRegionAccessor::MscRegionAccessor(MscRegionOperations *ops,
                                     uint32_t start_block, uint32_t num_blocks,
                                     fileid_t fileid)
: Accessor(fileid), _ops(ops)
{
    // NOTE: Block size assumption employed.

    _start = start_block * (offset_t) 512;
    _size = num_blocks * (offset_t) 512;
}

/* Return the size of the file. */

offset_t MscRegionAccessor::get_size()
{
    return _size;
}

/* Update the size of the file. */

void MscRegionAccessor::set_size(offset_t size)
{
    (void) size;
}

/* Data transfer helper methods. */

/* Fill the populated portion of a flexpage. */

void MscRegionAccessor::fill_populated(Flexpage *flexpage)
{
    offset_t filepos = flexpage->base_offset;
    offset_t populated_size = std::min(flexpage->size, _size - filepos);

    /* Tag the region with file state. */

    flexpage->region->fill(fileid, filepos);

    /* Fill the region with file content. */

    _ops->read(flexpage->base_addr, flexpage->base_addr_physical,
               _start + filepos, populated_size);

    /* Pad the flexpage with zero. */

    if (populated_size < flexpage->size)
        memset((void *) (flexpage->base_addr + populated_size), 0,
               flexpage->size - populated_size);
}

/* Flush the populated portion of a flexpage. */

void MscRegionAccessor::flush_populated(Flexpage *flexpage)
{
    offset_t filepos = flexpage->base_offset;
    offset_t populated_size = std::min(flexpage->size, _size - filepos);

    /* Remove the file state tag from the region. */

    flexpage->region->flush();

    /* Flush the file content. */

    _ops->write(flexpage->base_addr, flexpage->base_addr_physical,
                _start + filepos, populated_size);
}

// vim: tabstop=4 expandtab shiftwidth=4
